#include "../Framework/Framework.h"
#include "../Framework/Util/OpenGLModel.h"
#include "../Framework/Math/Color.h"

struct Light {
	Color color;
	Vertex position;

	Light(const Vertex &pos, const Color &col){
		position = pos;
		color = col;
	}
};

class MainApp : public OpenGLApp {
protected:
	OpenGLModel *model;

	Set <Light> lights;
	Set <TextureID> textures;
	ShaderID lightingVS, lightingFS;

	void drawObject();
public:
	void selectPixelFormat(PixelFormat &pixelFormat);

	bool init();
	bool exit();
	bool load();
	bool unload();
	bool drawFrame();
	
	void resetCamera();
};

void MainApp::resetCamera(){
	position = Vertex(0, 0, -600);
	Wx = 0;
	Wy = 0;
	Wz = 0;
}

void MainApp::selectPixelFormat(PixelFormat &pixelFormat){
	pixelFormat.redBits   = 8;
	pixelFormat.greenBits = 8;
	pixelFormat.blueBits  = 8;
	pixelFormat.alphaBits = 0;

	pixelFormat.depthBits = 24;
}

bool MainApp::init(){
	lights.add(Light(Vector(-800, -120,  190), Color(0, 0, 1)));
//	lights.add(Light(Vector(-800, -120, -190), Color(1, 0.7f, 0.1f)));

//	lights.add(Light(Vector(800, -120,  190), Color(1, 0.7f, 0.1f)));
	lights.add(Light(Vector(800, -120, -190), Color(0, 0, 1)));

	lights.add(Light(Vector( 160, -40,  70), Color(1, 0.7f, 0.1f)));
	lights.add(Light(Vector(-160, -40, -70), Color(1, 0.7f, 0.1f)));

	projection.loadProjectionMatrix(80 * PI / 180, 0.75f, 0.1f, 6000);
	return true;
}

bool MainApp::exit(){
	return true;
}

bool MainApp::load(){
	String supported, missing;
    if (GL_ARB_vertex_program)   supported += "GL_ARB_vertex_program\n";   else missing += "GL_ARB_vertex_program\n";
    if (GL_ARB_fragment_program) supported += "GL_ARB_fragment_program\n"; else missing += "GL_ARB_fragment_program\n";

    if (missing.getLength() > 0){
        ALERT("The following required OpenGL extensions are supported:\n\n" + supported +
				   "\nToo bad, these required OpenGL extensions aren't supported by the graphic card and driver you're using:\n\n" + missing +
                   "\nThis application will now exit.", "Missing extensions");
        return false;
    }

	textures.add(renderer->addTexture("../Models/DmTutorial/BrownFloor.dds"));
	textures.add(renderer->addTexture("../Models/DmTutorial/Demwall.dds"));
	textures.add(renderer->addTexture("../Models/DmTutorial/Demwall2.dds"));
	textures.add(renderer->addTexture("../Models/DmTutorial/BrownBase.dds"));
	textures.add(renderer->addTexture("../Models/DmTutorial/Angle1.dds"));
	textures.add(renderer->addTexture("../Models/DmTutorial/Angle2.dds"));
	textures.add(renderer->addTexture("../Models/DmTutorial/Demon1.dds"));
	textures.add(renderer->addTexture("../Models/DmTutorial/Demon2.dds"));
	textures.add(renderer->addTexture("../Models/DmTutorial/DemWin6.dds"));
	textures.add(renderer->addTexture("../Models/DmTutorial/Lampon2.dds"));

	model = new OpenGLModel();
	model->loadFromFile("../Models/DmTutorial/DmTutorial.hmdl");

	for (unsigned int i = 0; i < model->getBatchCount(); i++){
		model->getBatch(i)->addNormals();
		//((OpenGLBatch *) model->getBatch(0))->uploadToVertexBuffer();
	}

	textTexture = renderer->addTexture("../Textures/font2.dds");

	lightingVS = renderer->addVertexShader("Lighting.vsh");
	if (glGetError() != GL_NO_ERROR){
		ALERT((char *) glGetString(GL_PROGRAM_ERROR_STRING_ARB), "Error");
		return false;
	}

	lightingFS = renderer->addFragmentShader(GL_FRAGMENT_PROGRAM_ARB, "Lighting.fsh");
	if (glGetError() != GL_NO_ERROR){
		ALERT((char *) glGetString(GL_PROGRAM_ERROR_STRING_ARB), "Error");
		return false;
	}

	return true;
}

bool MainApp::unload(){
	delete model;
	textures.clear();

	return true;
}

void MainApp::drawObject(){
    glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf(modelView);

	glMatrixMode(GL_PROJECTION);
	glLoadMatrixf(projection);

	glEnable(GL_CULL_FACE);
	glCullFace(GL_BACK);
	glFrontFace(GL_CW);

//	for (unsigned int j = 2; j < 3; j++){
	for (unsigned int j = 0; j < lights.getSize(); j++){
		for (unsigned int i = 0; i < model->getBatchCount(); i++){
			renderer->setTextures(textures[i]);
			renderer->setVertexShader(lightingVS);
			renderer->setFragmentShader(lightingFS);
			if (j > 0) renderer->setBlending(GL_ONE, GL_ONE);
			renderer->apply();

			Vector lightPos = lights[j].position;
			glProgramLocalParameter4fARB(GL_VERTEX_PROGRAM_ARB, 0, position.x, position.y, position.z, 0);
			glProgramLocalParameter4fARB(GL_VERTEX_PROGRAM_ARB, 1, lightPos.x, lightPos.y, lightPos.z, 0);
			
			Vector lPos = lightPos - position;		
			glProgramLocalParameter4fARB (GL_FRAGMENT_PROGRAM_ARB, 0, lPos.x, lPos.y, lPos.z, 0);
			glProgramLocalParameter4fARB (GL_FRAGMENT_PROGRAM_ARB, 1, lightPos.x, lightPos.y, lightPos.z, 0);
			glProgramLocalParameter4fvARB(GL_FRAGMENT_PROGRAM_ARB, 2, lights[j].color);

			((OpenGLBatch *) model->getBatch(i))->draw();
		}
	}

	glDisable(GL_CULL_FACE);
}

bool MainApp::drawFrame(){
//	glClearColor(0, 1, 0, 1);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);

	glMatrixMode(GL_PROJECTION);
	glLoadMatrixf(projection);

	modelView.loadRotateZXY(-Wx, -Wy, -Wz);
	modelView.translate(-position);

    glMatrixMode(GL_MODELVIEW);
	glLoadMatrixf(modelView);

	drawObject();

	return true;
}

DECLARE_MAIN_APP(MainApp)
